#/////////////////////////////////////////////////////////////////////////////////
# convertSchToEdif.tcl 1.0
# 将一个目录下的所有 DSN 文件转换为 EDF 文件
#
#/////////////////////////////////////////////////////////////////////////////////

# 打印日志
proc tcl_log {str {fileId ""}} {
    DboState_WriteToSessionLog [DboTclHelper_sMakeCString $str]
    if {$fileId!="" && $fileId!="NULL"} {
        puts $fileId $str
    }
}
# 打印 error 信息
proc tcl_error {str {fileId ""}} {
    tcl_log "Error: $str" $fileId
}
# 打印 info 信息
proc tcl_info {str fileId} {
    tcl_log "Info: $str" $fileId
}

# 弹窗提示
proc tk_alert {str} {
    tk_messageBox -type ok -message $str -title "Message"
}

# 得到目录 path 下（包括子目录下）所有文件名
proc tcl_dir {path} {
    set result [list]
    if {"NULL"!=$path && ""!=$path} {
        # catach excption or error
        if {[catch {cd $path} err]} {
            tcl_error $err
            return $err
        }
    }
    foreach fileList [glob -nocomplain *] {
        if {[file isdirectory $fileList]} {
            set result [concat $result [tcl_dir "$path\\$fileList"]]
            cd ..
        } else {
            lappend result "$path\\$fileList"
        }
    }
    return $result
}

########################################
# tcl_dsn2edif 1.0
# 将 DSN 文件转为 EDIF 文件
#   inputDir    输入文件所在文件夹
#   outDir      保存文件的文件夹
#   configPath  EDIF 配置文件路径（可忽略，不清楚是什么作用）
########################################
proc tcl_dsn2edif {inputDir outDir configPath} {
    if {"NULL"==$inputDir || ""==$inputDir} {
        tk_alert "Source directory cannot be empty!"
        return -1
    }
    if {"NULL"==$outDir || ""==$outDir} {
        tk_alert "Output directory cannot be empty!"
        return -1
    }
    if { ![file exist $outDir] } {
        if {[catch {file mkdir $outDir} err]} {
            tcl_error $err
        }
    }

    # 创建 log 文件
    set logFile [open "$outDir\\log.txt" w+]
    tcl_log "Convert sch to edif" $logFile
    set date [clock format [clock seconds] -format {%Y/%m/%d %T}]
    tcl_log "Started on $date" $logFile

    # 当前文件夹所有文件路径
    set pathArr [tcl_dir $inputDir]
    set fileNum 0
    set successNum 0
    set failNum 0
    foreach path $pathArr {
        if { [regexp -nocase {\\([^\x00-\x1f:?\\\\/*"<>|]*)\.dsn$} $path fullMatch subMatch] } {
            incr fileNum
            tcl_info "converting: $path" $logFile

            set inputSubDir [string map [list $fullMatch ""] $path]
            set outSubDir [string map [list $inputDir $outDir] $inputSubDir]
            if { ![file exist $outSubDir] } {
                if {[catch {file mkdir $outSubDir} err]} {
                    tcl_error $err $logFile
                }
            }
            set outPath "$outSubDir\\$subMatch\.edf"

            if { [catch {XMATIC_CAP2EDIF $path $outPath $configPath} err] } {
                incr failNum
                tcl_error $err $logFile
                tcl_log "failed: $outPath" $logFile
            } else {
                if {[file exist $outPath]} {
                    incr successNum
                    tcl_info "successful: $outPath" $logFile
                } else {
                    incr failNum
                    tcl_log "failed: $outPath" $logFile
                }
            }
        }
    }
    tk_alert "This conversion was completed."
    tcl_log "total: $fileNum , successful: $successNum , failed: $failNum" $logFile
    close $logFile
}
