########################################
# tcl_olb2xml 1.0
# 将 OLB 文件转为 XML 文件
#   inputDir    输入文件所在文件夹
#   outDir      保存文件的文件夹
########################################
proc tcl_olb2xml {inputDir outDir} {
    if {"NULL"==$inputDir || ""==$inputDir} {
        tk_alert "Source directory cannot be empty!"
        return -1
    }
    if {"NULL"==$outDir || ""==$outDir} {
        tk_alert "Output directory cannot be empty!"
        return -1
    }
    if { ![file exist $outDir] } {
        if {[catch {file mkdir $outDir} err]} {
            tcl_error $err
        }
    }

    # 创建 log 文件
    set logFile [open "$outDir\\log.txt" w+]
    tcl_log "Convert schlib to xml" $logFile
    set date [clock format [clock seconds] -format {%Y/%m/%d %T}]
    tcl_log "Started on $date" $logFile

    set fileNum 0
    set successNum 0
    set failNum 0
    set pathArr [tcl_dir $inputDir]
    foreach path $pathArr {
        if { [regexp -nocase {\\([^\x00-\x1f:?\\\\/*"<>|]*)\.olb$} $path fullMatch subMatch] } {
            incr fileNum
            tcl_info "converting: $path" $logFile

            set inputSubDir [string map [list $fullMatch ""] $path]
            set outSubDir [string map [list $inputDir $outDir] $inputSubDir]
            if { ![file exist $outSubDir] } {
                if {[catch {file mkdir $outSubDir} err]} {
                    tcl_error $err $logFile
                }
            }
            set outPath "$outSubDir\\$subMatch\.xml"

            if { [catch {XMATIC_OLB2XML $path $outPath} err] } {
                incr failNum
                tcl_error $err $logFile
                tcl_log "failed: $outPath" $logFile
            } else {
                if {[file exist $outPath]} {
                    incr successNum
                    tcl_info "successful: $outPath" $logFile
                } else {
                    incr failNum
                    tcl_log "failed: $outPath" $logFile
                }
            }

            __appendPcbFootprint2XML $path $outPath $logFile
        }
    }
    tk_alert "This conversion was completed."
    tcl_log "total: $fileNum , successful: $successNum , failed: $failNum" $logFile
    close $logFile
}

# OrCad 16.6 低版本导出的 XML 文件缺少 pcbFootprint 属性
# 这里用正则表达式匹配增加该属性到 XML 文件中
proc __appendPcbFootprint2XML {olbPath xmlPath logFile} {
    set lSession [DboTclHelper_sCreateSession]
    set lLibName [DboTclHelper_sMakeCString $olbPath]
    set lStatus [DboState]
    # 获取到 lib
    set lLib [$lSession GetLib $lLibName $lStatus]
    set lNullObj NULL

    if { $lLib == $lNullObj} {
        tcl_error "Open lib Failed" $logFile
        return
    }

    if { ![file exist $xmlPath] } {
        tcl_error "Open xml Failed" $logFile
        return
    }
    # 打开 XML 文件；这里不能用 w，否则读取到的文件内容为空
    set xmlFile [open $xmlPath r]
    # 读取 XML 内容
    set xmlContent [read $xmlFile]
    close $xmlFile
    # 从中匹配 pcbFootprint 字符，匹配到了就不进行正则替换
    if { [regexp {pcbFootprint=} $xmlContent] } {
        tcl_info "Exist pcbFootprint property." $logFile
        return
    } else {
        tcl_info "append pcbFootprint property to $xmlPath" $logFile
    }

    # NewPackagesIter 迭代器
    set pkgIter [$lLib NewPackagesIter $lStatus]
    set pPkg [$pkgIter NextPackage $lStatus]
    set lNullObj NULL
    while {$pPkg!=$lNullObj} {
        set pActualPkgName [DboTclHelper_sMakeCString]
        set lCachedPCBFootprintNameStr [DboTclHelper_sMakeCString]
        # 符号名
        $pPkg GetName $pActualPkgName
        set pkgName [DboTclHelper_sGetConstCharPtr $pActualPkgName]
        # tcl_log "$pkgName" $logFile

        # 封装名
        $pPkg GetPCBFootprint $lCachedPCBFootprintNameStr
        set pkgPCBFootprintNameStr [DboTclHelper_sGetConstCharPtr $lCachedPCBFootprintNameStr]
        # tcl_log "$pkgPCBFootprintNameStr" $logFile

        # 替换
        # e.g. (name="R0402_1" )--> (name="R0402_1" pcbFootprint="R0402" )
        set xmlContent [string map [list "name=\"$pkgName\" " "name=\"$pkgName\" pcbFootprint=\"$pkgPCBFootprintNameStr\" "] $xmlContent]

        set pPkg [$pkgIter NextPackage $lStatus]
    }
    delete_DboLibPackagesIter $pkgIter

    # tcl_log $xmlContent $logFile
    set xmlFile [open $xmlPath w+]
    puts $xmlFile $xmlContent
    close $xmlFile
}
